/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelspacerclass.c,v 1.1 2006/12/20 13:34:15 fab Exp $
 */

#include "ambient.h"

/* public */
#include <proto/cybergraphics.h>

/* private */
#include "mui_func.h"
#include "rexx.h"
#include "command.h"
#include "contextmenu.h"
#include "name.h"
#include "prefs.h"
#include "threads.h"
#include "file_func.h"
#include "legacy.h"
#include "prefs_advanced.h"
#include "gfx_analyze.h"

struct Data {
	ULONG  distance;
	ULONG  size;
	APTR   cmenu;
	STRPTR help;

	ULONG  brightness_threshold;  
	BOOL   brightness_checked;    
};

static void doset(APTR obj, struct Data *data, struct TagItem *tags)
{
	ULONG update = 0;

	FORTAG(tags)
	{
		case MA_Panel_Distance:
			data->distance = tag->ti_Data;
			break;

		case MA_Panelgroup_Locked:
			MUI_Redraw(obj, MADF_DRAWOBJECT);
			break;

		case MA_Panelgroup_ImageSpec:  
			data->brightness_checked = FALSE; /* invalidate cache, see MUIM_Draw */
			MUI_Redraw(obj, MADF_DRAWOBJECT);
			break;
	}
	NEXTTAG 
}


DEFNEW
{
	struct Data *data;
	APTR strip;

	obj = DoSuperNew(cl, obj,
		MUIA_Frame, MUIV_Frame_None,
		MUIA_InnerLeft,   1,
		MUIA_InnerRight,  1,
		MUIA_InnerTop,    1,
		MUIA_InnerBottom, 1,
		MUIA_ContextMenu, 1,
		TAG_DONE
	);

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);

	data->help  = NULL;
	data->size  = _aprefs(panelspacersize);
	
	data->brightness_checked   = FALSE;
	data->brightness_threshold = 255; /* default to bright bgs */

    doset(obj, data, INITTAGS);

	return ((ULONG)obj);
}


DEFDISP
{
	struct applet_node *n, *nn;
	GETDATA;

	if (data->help)
	{
		name_delete(data->help);
	}

	return (DOSUPER);
}


DEFMMETHOD(AskMinMax)
{
	ULONG size;
	ULONG horiz;
    GETDATA;

	DOSUPER;

	size  = getv(_parent(obj), MA_Panelgroup_Size);
	horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

	if (horiz)
	{
		if (msg->MinMaxInfo->MinWidth < data->size)
		{
			msg->MinMaxInfo->MinWidth = data->size;
			msg->MinMaxInfo->MaxWidth = data->size;
		}
		msg->MinMaxInfo->MinHeight = size;
		msg->MinMaxInfo->MaxHeight = size;
	}
	else
	{
		if (msg->MinMaxInfo->MinHeight < data->size)
		{
			msg->MinMaxInfo->MinHeight = data->size;
			msg->MinMaxInfo->MaxHeight = data->size;
		}
		msg->MinMaxInfo->MinWidth = size;
		msg->MinMaxInfo->MaxWidth = size;
	}
	return (0);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Panel_Type:
			*msg->opg_Storage = MV_Panel_Type_Spacer;
			return (TRUE);
			break;

		case MA_Panel_Distance:
			{
				GETDATA;
				*msg->opg_Storage = data->distance;
			}
			return (TRUE);
			break;

		case MA_Panel_Properties:
            *msg->opg_Storage = TRUE;
			return (TRUE);
			break;
	}

	return (DOSUPER);
}


DEFSET
{
	GETDATA;

	doset(obj, data, INITTAGS);

	return (DOSUPER);
}


DEFMMETHOD(Draw)
{
	GETDATA;

	DOSUPER;

	if (msg->flags & MADF_DRAWOBJECT && 
	    (!getv(_parent(obj), MA_Panelgroup_Locked)))
	{
		struct RastPort *rp = _rp(obj);

        ULONG mleft         = _mleft(obj);
		ULONG mtop          = _mtop(obj);
		ULONG mright        = _mright(obj);
		ULONG mbottom       = _mbottom(obj);

		ULONG x, y;
		ULONG colour = 0xFFFFFFFF;
		BOOL ff;

		/*   check if we have to draw white or black lines; cache the value 
		 */
		if (data->brightness_checked == FALSE)
		{
			data->brightness_checked   = TRUE;
			data->brightness_threshold = gfx_analyze_average_brightness(rp, mleft, mtop, _mwidth(obj), _mheight(obj));
		}

		if (data->brightness_threshold > 128)
		{
			colour = 0xFF000000;
		}

		for (x = mleft + 1, ff = TRUE; x < mright; x++)
		{
			if (ff) 
			{ 
				WriteRGBPixel(rp, x, mtop,    colour); 
				WriteRGBPixel(rp, x, mbottom, colour);
			}

			ff = !ff;
		}

		for (y = mtop + 1, ff = TRUE; y < mbottom; y++)
		{
			if (ff) 
			{ 
				WriteRGBPixel(rp, mleft,  y, colour); 
				WriteRGBPixel(rp, mright, y, colour);
			}

			ff = !ff;
		}
	}

	return 0;
}

DEFMMETHOD(ContextMenuAdd)
{
	ULONG flags = 0;
	GETDATA;

	if (!getv(_parent(obj), MA_Panelgroup_Locked))
		flags |= AS_PANEL_UNLOCKED;

	data->cmenu	= contextmenu_build_simple(CM_PANELSPACER, flags);

	if (data->cmenu)
		DoMethod(msg->menustrip, OM_ADDMEMBER, data->cmenu);

	return (DOSUPER);
}


DEFMMETHOD(ContextMenuChoice)
{
	GETDATA;
	struct command_menu *cm;
	ULONG rc = 0;

	if (data->cmenu && DoMethod(data->cmenu, MUIM_FindObject, msg->item))
	{
		if ( ((cm = (struct command_menu *)getv(msg->item, MA_Menuitem_Command))) )
		{
			if (cm->name && *cm->name)
				execute_command(_parent(obj), cm->type, cm->args, NULL);
		}
	}
	else
		rc = DOSUPER;

	data->cmenu = NULL;

	return (rc);
}


DEFSMETHOD(Panel_SaveConfig)
{
	APTR pl, pi; /* list, item */
	GETDATA;
	ASSERT(msg->pctx);

	if (!(pl = prefspool_item_get(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL)))
	{
		pl = prefspool_item_add(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL);
	}


	if (pl)
	{
		if (!(pi = prefspool_item_get(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL)))
		{
			pi = prefspool_item_add(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL);
		}
		if (pi)
		{
			setprefslong_lp(msg->pctx, pi, DSI_LISTPOOL_PANEL_DISTANCE, data->distance);
		}
	}
	
	return (0);
}


BEGINMTABLE
DECNEW
DECDISP
DECGET
DECSET
DECMMETHOD(AskMinMax)
DECMMETHOD(Draw)
DECMMETHOD(ContextMenuChoice)
DECMMETHOD(ContextMenuAdd)
DECSMETHOD(Panel_SaveConfig)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Area, panelspacerclass)

